"""Collection of tests to ensure correct handling of invalid input."""

from fastapi.testclient import TestClient
import pytest

from ..main import app
from ..constants import SUPABASE_TEST_USER_ID


@pytest.fixture(scope="module")
def invalid_client():
    """Client used to call the app."""
    return TestClient(app)


@pytest.mark.parametrize(
    "start,preferences,status_code",
    [
        # Invalid case: no preferences at all.
        ([48.8566, 2.3522], {}, 422),

        # Invalid cases: incomplete preferences.
        ([48.084588, 7.280405], {"sightseeing": {"type": "nature", "score": 5}, # no shopping
                                 "nature": {"type": "nature", "score": 5},
                                 }, 422),
        ([48.084588, 7.280405], {"sightseeing": {"type": "nature", "score": 5}, # no nature
                                 "shopping": {"type": "shopping", "score": 5},
                                 }, 422),
        ([48.084588, 7.280405], {"nature": {"type": "nature", "score": 5},      # no sightseeing
                                 "shopping": {"type": "shopping", "score": 5},
                                 }, 422),

        # Invalid cases: unexisting coords
        ([91, 181], {"sightseeing": {"type": "nature", "score": 5},
                     "nature": {"type": "nature", "score": 5},
                     "shopping": {"type": "shopping", "score": 5},
                    }, 422),
        ([-91, 181], {"sightseeing": {"type": "nature", "score": 5},
                     "nature": {"type": "nature", "score": 5},
                     "shopping": {"type": "shopping", "score": 5},
                    }, 422),
        ([91, -181], {"sightseeing": {"type": "nature", "score": 5},
                     "nature": {"type": "nature", "score": 5},
                     "shopping": {"type": "shopping", "score": 5},
                    }, 422),
        ([-91, -181], {"sightseeing": {"type": "nature", "score": 5},
                     "nature": {"type": "nature", "score": 5},
                     "shopping": {"type": "shopping", "score": 5},
                    }, 422),
    ]
)
def test_input(invalid_client, start, preferences, status_code):   # pylint: disable=redefined-outer-name
    """
    Test new trip creation with different sets of preferences and locations.
    """
    response = invalid_client.post(
        "/trip/new",
        json={
            "user_id": SUPABASE_TEST_USER_ID,
            "preferences": preferences,
            "start": start
        }
    )
    assert response.status_code == status_code



@pytest.mark.parametrize(
    "user_id,status_code",
    [
        # No user id :
        ({}, 422),
        ("invalid_user_id", 400),
        # ("12345678-1234-5678-1234-567812345678", 406)
    ]
)
def test_input(invalid_client, user_id, status_code):   # pylint: disable=redefined-outer-name
    """
    Test new trip creation with invalid user ID.
    """
    response = invalid_client.post(
        "/trip/new",
        json={
            "user_id": user_id,
            "preferences": {"sightseeing": {"type": "sightseeing", "score": 5},
                "nature": {"type": "nature", "score": 0},
                "shopping": {"type": "shopping", "score": 0},
                "max_time_minute": 20,
                "detour_tolerance_minute": 0},
            "start": [48.084588, 7.280405]
        }
    )
    assert response.status_code == status_code
